# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise.plugins.messagebird import NotifyMessageBird

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "msgbird://",
        {
            # No hostname/apikey specified
            "instance": TypeError,
        },
    ),
    (
        "msgbird://{}/abcd".format("a" * 25),
        {
            # invalid characters in source phone number
            "instance": TypeError,
        },
    ),
    (
        "msgbird://{}/123".format("a" * 25),
        {
            # invalid source phone number
            "instance": TypeError,
        },
    ),
    (
        "msgbird://{}/15551232000".format("a" * 25),
        {
            # target phone number becomes who we text too; all is good
            "instance": NotifyMessageBird,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "msgbird://a...a/15551232000",
        },
    ),
    (
        "msgbird://{}/15551232000/abcd".format("a" * 25),
        {
            # valid credentials
            "instance": NotifyMessageBird,
            # Since there are no targets specified we expect a False return on
            # send()
            "notify_response": False,
        },
    ),
    (
        "msgbird://{}/15551232000/123".format("a" * 25),
        {
            # valid credentials
            "instance": NotifyMessageBird,
            # Since there are no targets specified we expect a False return on
            # send()
            "notify_response": False,
        },
    ),
    (
        "msgbird://{}/?from=15551233000&to=15551232000".format("a" * 25),
        {
            # reference to to= and from=
            "instance": NotifyMessageBird,
        },
    ),
    (
        "msgbird://{}/15551232000".format("a" * 25),
        {
            "instance": NotifyMessageBird,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "msgbird://{}/15551232000".format("a" * 25),
        {
            "instance": NotifyMessageBird,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "msgbird://{}/15551232000".format("a" * 25),
        {
            "instance": NotifyMessageBird,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_messagebird_urls():
    """NotifyTemplate() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_messagebird_edge_cases(mock_post):
    """NotifyMessageBird() Edge Cases."""

    # Prepare our response
    response = requests.Request()
    response.status_code = requests.codes.ok

    # Prepare Mock
    mock_post.return_value = response

    # Initialize some generic (but valid) tokens
    # authkey = '{}'.format('a' * 24)
    source = "+1 (555) 123-3456"

    # No apikey specified
    with pytest.raises(TypeError):
        NotifyMessageBird(apikey=None, source=source)
    with pytest.raises(TypeError):
        NotifyMessageBird(apikey="     ", source=source)



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise.plugins.flock import NotifyFlock

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "flock://",
        {
            "instance": TypeError,
        },
    ),
    # An invalid url
    (
        "flock://:@/",
        {
            "instance": TypeError,
        },
    ),
    # Provide a token
    (
        "flock://%s" % ("t" * 24),
        {
            "instance": NotifyFlock,
        },
    ),
    # Image handling
    (
        "flock://%s?image=True" % ("t" * 24),
        {
            "instance": NotifyFlock,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "flock://t...t",
        },
    ),
    (
        "flock://%s?image=False" % ("t" * 24),
        {
            "instance": NotifyFlock,
        },
    ),
    (
        "flock://%s?image=True" % ("t" * 24),
        {
            "instance": NotifyFlock,
            # Run test when image is set to True, but one couldn't actually be
            # loaded from the Asset Object.
            "include_image": False,
        },
    ),
    # Test to=
    (
        "flock://{}?to=u:{}&format=markdown".format("i" * 24, "u" * 12),
        {
            "instance": NotifyFlock,
        },
    ),
    # Provide markdown format
    (
        "flock://%s?format=markdown" % ("i" * 24),
        {
            "instance": NotifyFlock,
        },
    ),
    # Provide text format
    (
        "flock://%s?format=text" % ("i" * 24),
        {
            "instance": NotifyFlock,
        },
    ),
    # Native URL Support, take the slack URL and still build from it
    (
        "https://api.flock.com/hooks/sendMessage/{}/".format("i" * 24),
        {
            "instance": NotifyFlock,
        },
    ),
    # Native URL Support with arguments
    (
        "https://api.flock.com/hooks/sendMessage/{}/?format=markdown".format(
            "i" * 24
        ),
        {
            "instance": NotifyFlock,
        },
    ),
    # Bot API presumed if one or more targets are specified
    # Provide markdown format
    (
        "flock://{}/u:{}?format=markdown".format("i" * 24, "u" * 12),
        {
            "instance": NotifyFlock,
        },
    ),
    # Bot API presumed if one or more targets are specified
    # Provide text format
    (
        "flock://{}/u:{}?format=html".format("i" * 24, "u" * 12),
        {
            "instance": NotifyFlock,
        },
    ),
    # Bot API presumed if one or more targets are specified
    # u: is optional
    (
        "flock://{}/{}?format=text".format("i" * 24, "u" * 12),
        {
            "instance": NotifyFlock,
        },
    ),
    # Bot API presumed if one or more targets are specified
    # Multi-entries
    (
        "flock://{}/g:{}/u:{}?format=text".format(
            "i" * 24, "g" * 12, "u" * 12
        ),
        {
            "instance": NotifyFlock,
        },
    ),
    # Bot API presumed if one or more targets are specified
    # Multi-entries using @ for user and # for channel
    (
        "flock://{}/#{}/@{}?format=text".format("i" * 24, "g" * 12, "u" * 12),
        {
            "instance": NotifyFlock,
        },
    ),
    # Bot API presumed if one or more targets are specified
    # has bad entry
    (
        "flock://{}/g:{}/u:{}?format=text".format(
            "i" * 24, "g" * 12, "u" * 10
        ),
        {
            "instance": NotifyFlock,
        },
    ),
    # Invalid user/group defined
    (
        "flock://%s/g:/u:?format=text" % ("i" * 24),
        {
            "instance": TypeError,
        },
    ),
    # we don't focus on the invalid length of the user/group fields.
    # As a result, the following will load and pass the data upstream
    (
        "flock://{}/g:{}/u:{}?format=text".format(
            "i" * 24, "g" * 14, "u" * 10
        ),
        {
            # We will still instantiate the object
            "instance": NotifyFlock,
        },
    ),
    # Error Testing
    (
        "flock://{}/g:{}/u:{}?format=text".format(
            "i" * 24, "g" * 12, "u" * 10
        ),
        {
            "instance": NotifyFlock,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "flock://%s/" % ("t" * 24),
        {
            "instance": NotifyFlock,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "flock://%s/" % ("t" * 24),
        {
            "instance": NotifyFlock,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "flock://%s/" % ("t" * 24),
        {
            "instance": NotifyFlock,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_flock_urls():
    """NotifyFlock() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.get")
@mock.patch("requests.post")
def test_plugin_flock_edge_cases(mock_post, mock_get):
    """NotifyFlock() Edge Cases."""

    # Initializes the plugin with an invalid token
    with pytest.raises(TypeError):
        NotifyFlock(token=None)
    # Whitespace also acts as an invalid token value
    with pytest.raises(TypeError):
        NotifyFlock(token="   ")



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import requests

from apprise import Apprise, NotifyType
from apprise.plugins.africas_talking import NotifyAfricasTalking

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "atalk://",
        {
            # Instantiated but no auth, so no notification can happen
            "instance": TypeError,
        },
    ),
    (
        "atalk://:@/",
        {
            # invalid auth
            "instance": TypeError
        },
    ),
    (
        "atalk://user@^/",
        {
            # invalid apikey
            "instance": TypeError
        },
    ),
    (
        "atalk://user@apikey/{}".format("3" * 5),
        {
            # invalid nubmer provided
            "instance": NotifyAfricasTalking,
            # Expected notify() response because we have no one to notify
            "notify_response": False,
        },
    ),
    (
        "atalk://user@apikey/123/{}/abcd/+{}".format("3" * 11, "4" * 11),
        {
            # includes a few invalid bits of info
            "instance": NotifyAfricasTalking,
            "privacy_url": "atalk://user@a...y/33333333333/+44444444444",
        },
    ),
    (
        "atalk://user@apikey/+{}?batch=y".format("4" * 11),
        {
            "instance": NotifyAfricasTalking,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "atalk://user@a...y/+44444444444",
        },
    ),
    (
        "atalk://user@apikey/+{}?mode=invalid".format("4" * 11),
        {"instance": TypeError},
    ),
    (
        "atalk://user@apikey/+{}?mode=s".format("4" * 11),
        {
            # S will match the sandbox
            "instance": NotifyAfricasTalking,
        },
    ),
    (
        "atalk://user@apikey/+{}?mode=PREM".format("4" * 11),
        {
            # PREM will match premium (not case sensitive)
            "instance": NotifyAfricasTalking,
        },
    ),
    (
        "atalk://{}?apikey=key&user=user&from=FROMUSER".format("1" * 11),
        {
            # use get args to acomplish the same thing
            "instance": NotifyAfricasTalking,
        },
    ),
    (
        "atalk://_?user=user&to={},{}&key={}&from={}".format(
            "1" * 11, "2" * 11, "b" * 10, "5" * 13
        ),
        {
            # use get args to acomplish the same thing
            "instance": NotifyAfricasTalking,
        },
    ),
    (
        "atalk://user@apikey/{}/".format("1" * 11),
        {
            "instance": NotifyAfricasTalking,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "atalk://user@apikey/{}/".format("1" * 11),
        {
            "instance": NotifyAfricasTalking,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_atalk_urls():
    """NotifyTemplate() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_atalk_edge_cases(mock_post):
    """NotifyAfricasTalking() Edge Cases."""

    # Initialize some generic (but valid) tokens
    apikey = "my-api-key"
    appuser = "my-app-user"
    targets = [
        "+1(555) 123-1234",
        "1555 5555555",
        # A garbage entry
        "12",
    ]

    # Prepare our response
    response = requests.Request()
    response.status_code = requests.codes.ok

    # Prepare Mock
    mock_post.return_value = response

    # Instantiate our object
    obj = Apprise.instantiate(
        "atalk://{}@{}/{}?batch=n".format(appuser, apikey, "/".join(targets))
    )

    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )

    # We know there are 2 (valid) targets
    assert len(obj) == 2

    # Test our call count
    assert mock_post.call_count == 2

    # Test
    details = mock_post.call_args_list[0]
    headers = details[1]["headers"]
    assert headers["apiKey"] == apikey
    payload = details[1]["data"]
    assert payload["username"] == appuser
    assert payload["from"] == "AFRICASTKNG"
    assert payload["to"] == "+15551231234"
    assert payload["message"] == "title\r\nbody"

    details = mock_post.call_args_list[1]
    headers = details[1]["headers"]
    assert headers["apiKey"] == apikey
    payload = details[1]["data"]
    assert payload["username"] == appuser
    assert payload["from"] == "AFRICASTKNG"
    assert payload["to"] == "15555555555"
    assert payload["message"] == "title\r\nbody"

    # Verify our URL looks good
    assert obj.url().startswith(
        "atalk://{}@{}/{}".format(
            appuser, apikey, "/".join(["+15551231234", "15555555555"])
        )
    )

    assert "batch=no" in obj.url()

    # Reset our mock object
    mock_post.reset_mock()

    # With our batch in place, our calculations are different
    # Testing URL restructuring here as well where phone # is found
    # in host
    obj = Apprise.instantiate(
        "atalk://{}?user={}&apikey={}&batch=y&from=TEST".format(
            "/".join(targets), appuser, apikey
        )
    )

    # 2 phones were loaded but counted as 1 due to batch flag
    assert len(obj) == 1

    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )

    # Test our call count (batched into 1)
    assert mock_post.call_count == 1

    details = mock_post.call_args_list[0]
    headers = details[1]["headers"]
    assert headers["apiKey"] == apikey
    payload = details[1]["data"]
    assert payload["username"] == appuser
    assert payload["from"] == "TEST"
    assert payload["to"] == "+15551231234,15555555555"
    assert payload["message"] == "title\r\nbody"



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise.plugins.pushed import NotifyPushed

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "pushed://",
        {
            "instance": TypeError,
        },
    ),
    # Application Key Only
    (
        "pushed://%s" % ("a" * 32),
        {
            "instance": TypeError,
        },
    ),
    # Invalid URL
    (
        "pushed://:@/",
        {
            "instance": TypeError,
        },
    ),
    # Application Key+Secret
    (
        "pushed://{}/{}".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
        },
    ),
    # Application Key+Secret + channel
    (
        "pushed://{}/{}/#channel/".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
        },
    ),
    # Application Key+Secret + channel (via to=)
    (
        "pushed://{}/{}?to=channel".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "pushed://a...a/****/",
        },
    ),
    # Application Key+Secret + dropped entry
    (
        "pushed://{}/{}/dropped_value/".format("a" * 32, "a" * 64),
        {
            # No entries validated is a fail
            "instance": TypeError,
        },
    ),
    # Application Key+Secret + 2 channels
    (
        "pushed://{}/{}/#channel1/#channel2".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
        },
    ),
    # Application Key+Secret + User Pushed ID
    (
        "pushed://{}/{}/@ABCD/".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
        },
    ),
    # Application Key+Secret + 2 devices
    (
        "pushed://{}/{}/@ABCD/@DEFG/".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
        },
    ),
    # Application Key+Secret + Combo
    (
        "pushed://{}/{}/@ABCD/#channel".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
        },
    ),
    # ,
    (
        "pushed://{}/{}".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "pushed://{}/{}".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "pushed://{}/{}".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
    (
        "pushed://{}/{}".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "pushed://{}/{}".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "pushed://{}/{}/#channel".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "pushed://{}/{}/@user".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "pushed://{}/{}".format("a" * 32, "a" * 64),
        {
            "instance": NotifyPushed,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_pushed_urls():
    """NotifyPushed() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.get")
@mock.patch("requests.post")
def test_plugin_pushed_edge_cases(mock_post, mock_get):
    """NotifyPushed() Edge Cases."""

    # Chat ID
    recipients = "@ABCDEFG, @DEFGHIJ, #channel, #channel2"

    # Some required input
    app_key = "ABCDEFG"
    app_secret = "ABCDEFG"

    # Prepare Mock
    mock_get.return_value = requests.Request()
    mock_post.return_value = requests.Request()
    mock_post.return_value.status_code = requests.codes.ok
    mock_get.return_value.status_code = requests.codes.ok

    # No application Key specified
    with pytest.raises(TypeError):
        NotifyPushed(
            app_key=None,
            app_secret=app_secret,
            recipients=None,
        )

    with pytest.raises(TypeError):
        NotifyPushed(
            app_key="  ",
            app_secret=app_secret,
            recipients=None,
        )
    # No application Secret specified
    with pytest.raises(TypeError):
        NotifyPushed(
            app_key=app_key,
            app_secret=None,
            recipients=None,
        )

    with pytest.raises(TypeError):
        NotifyPushed(
            app_key=app_key,
            app_secret="   ",
        )

    # recipients list set to (None) is perfectly fine; in this case it will
    # notify the App
    obj = NotifyPushed(
        app_key=app_key,
        app_secret=app_secret,
        recipients=None,
    )
    assert isinstance(obj, NotifyPushed) is True
    assert len(obj.channels) == 0
    assert len(obj.users) == 0

    obj = NotifyPushed(
        app_key=app_key,
        app_secret=app_secret,
        targets=recipients,
    )
    assert isinstance(obj, NotifyPushed) is True
    assert len(obj.channels) == 2
    assert len(obj.users) == 2

    # Prepare Mock to fail
    mock_post.return_value.status_code = requests.codes.internal_server_error
    mock_get.return_value.status_code = requests.codes.internal_server_error



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging

from helpers import AppriseURLTester
import pytest
import requests

from apprise.plugins.ryver import NotifyRyver

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "ryver://",
        {
            "instance": TypeError,
        },
    ),
    (
        "ryver://:@/",
        {
            "instance": TypeError,
        },
    ),
    (
        "ryver://apprise",
        {
            # Just org provided (no token)
            "instance": TypeError,
        },
    ),
    (
        "ryver://apprise/ckhrjW8w672m6HG?mode=invalid",
        {
            # invalid mode provided
            "instance": TypeError,
        },
    ),
    (
        "ryver://x/ckhrjW8w672m6HG?mode=slack",
        {
            # Invalid org
            "instance": TypeError,
        },
    ),
    (
        "ryver://apprise/ckhrjW8w672m6HG?mode=slack",
        {
            # No username specified; this is still okay as we use whatever
            # the user told the webhook to use; set our slack mode
            "instance": NotifyRyver,
        },
    ),
    (
        "ryver://apprise/ckhrjW8w672m6HG?mode=ryver",
        {
            # No username specified; this is still okay as we use whatever
            # the user told the webhook to use; set our ryver mode
            "instance": NotifyRyver,
        },
    ),
    # Legacy webhook mode setting:
    # Legacy webhook mode setting:
    (
        "ryver://apprise/ckhrjW8w672m6HG?webhook=slack",
        {
            # No username specified; this is still okay as we use whatever
            # the user told the webhook to use; set our slack mode
            "instance": NotifyRyver,
        },
    ),
    (
        "ryver://apprise/ckhrjW8w672m6HG?webhook=ryver",
        {
            # No username specified; this is still okay as we use whatever
            # the user told the webhook to use; set our ryver mode
            "instance": NotifyRyver,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "ryver://apprise/c...G",
        },
    ),
    # Support Native URLs
    (
        "https://apprise.ryver.com/application/webhook/ckhrjW8w672m6HG",
        {
            "instance": NotifyRyver,
        },
    ),
    # Support Native URLs with arguments
    (
        (
            "https://apprise.ryver.com/application/webhook/ckhrjW8w672m6HG"
            "?webhook=ryver"
        ),
        {
            "instance": NotifyRyver,
        },
    ),
    (
        "ryver://caronc@apprise/ckhrjW8w672m6HG",
        {
            "instance": NotifyRyver,
            # don't include an image by default
            "include_image": False,
        },
    ),
    (
        "ryver://apprise/ckhrjW8w672m6HG",
        {
            "instance": NotifyRyver,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "ryver://apprise/ckhrjW8w672m6HG",
        {
            "instance": NotifyRyver,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "ryver://apprise/ckhrjW8w672m6HG",
        {
            "instance": NotifyRyver,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_ryver_urls():
    """NotifyRyver() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


def test_plugin_ryver_edge_cases():
    """NotifyRyver() Edge Cases."""

    # No token
    with pytest.raises(TypeError):
        NotifyRyver(organization="abc", token=None)

    with pytest.raises(TypeError):
        NotifyRyver(organization="abc", token="  ")

    # No organization
    with pytest.raises(TypeError):
        NotifyRyver(organization=None, token="abc")

    with pytest.raises(TypeError):
        NotifyRyver(organization="  ", token="abc")



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise import Apprise, NotifyType
from apprise.plugins.exotel import NotifyExotel

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "exotel://",
        {
            # No Account SID specified
            "instance": TypeError,
        },
    ),
    (
        "exotel://:@/",
        {
            # invalid Auth token
            "instance": TypeError,
        },
    ),
    (
        "exotel://{}@12345678".format("a" * 32),
        {
            # Just sid provided
            "instance": TypeError,
        },
    ),
    (
        "exotel://{}:{}@_".format("a" * 32, "b" * 32),
        {
            # sid and token provided but invalid from
            "instance": TypeError,
        },
    ),
    (
        "exotel://{}:{}@/%20".format("a" * 32, "b" * 32),
        {
            # sid and token provided but no from
            "instance": TypeError,
        },
    ),
    (
        "exotel://{}:{}@{}".format("a" * 32, "b" * 32, "3" * 8),
        {
            # sid and token provided and from but invalid from no
            "instance": TypeError,
        },
    ),
    (
        "exotel://{}:{}@{}".format("a" * 32, "b" * 32, "3" * 9),
        {
            # sid and token provided and from
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://{}:{}@EXOTEL/{}".format("a" * 32, "b" * 32, "3" * 9),
        {
            # sid and token provided with a sender ID source
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://{}:{}@600123/{}".format("a" * 32, "b" * 32, "3" * 9),
        {
            # sid and token provided with a numeric sender ID source
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://{}:{}@{}/123/{}/abcd/".format(
            "a" * 32, "b" * 32, "3" * 11, "9" * 15
        ),
        {
            # valid everything but target numbers
            "instance": NotifyExotel,
            # Since the targets are invalid, we'll fail to send()
            "notify_response": False,
        },
    ),
    (
        "exotel://{}:{}@12345/{}".format("a" * 32, "b" * 32, "4" * 11),
        {
            # using short-code (5 characters) is not supported
            "instance": TypeError,
        },
    ),
    (
        "exotel://{}:{}@{}".format("a" * 32, "b" * 32, "5" * 11),
        {
            # using phone no with no target - we text ourselves in
            # this case
            "instance": NotifyExotel,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "exotel://****:b...b@55555555555/",
        },
    ),
    (
        "exotel://{}:{}@{}?batch=yes".format("a" * 32, "b" * 32, "5" * 11),
        {
            # Test batch flag
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://{}:{}@{}?batch=no".format("a" * 32, "b" * 32, "5" * 11),
        {
            # Test batch flag
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # use get args to accomplish the same thing
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&unicode=Yes".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test unicode flag
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&unicode=no".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test unicode flag
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&region=us".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test region flag (Us)
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&region=in".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test region flag (India)
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&region=invalid".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test region flag Invalid
            "instance": TypeError,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&priority=normal".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test region flag (Us)
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&priority=high".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test region flag (India)
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&priority=invalid".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # Test region flag Invalid
            "instance": TypeError,
        },
    ),
    (
        "exotel://_?sid={}&token={}&source={}".format(
            "a" * 32, "b" * 32, "5" * 11
        ),
        {
            # use get args to accomplish the same thing (use source
            # instead of from)
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://_?sid={}&token={}&from={}&to={}".format(
            "a" * 32, "b" * 32, "5" * 11, "7" * 13
        ),
        {
            # use to=
            "instance": NotifyExotel,
        },
    ),
    (
        "exotel://{}:{}@{}".format("a" * 32, "b" * 32, "6" * 11),
        {
            "instance": NotifyExotel,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "exotel://{}:{}@{}".format("a" * 32, "b" * 32, "6" * 11),
        {
            "instance": NotifyExotel,
            # Throws a series of connection and transfer exceptions when
            # this flag is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_exotel_urls():
    """NotifyExotel() Apprise URLs."""

    assert NotifyExotel.setup_url == "https://appriseit.com/services/exotel/"

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_exotel_edge_cases(mock_post):
    """NotifyExotel() Edge Cases."""

    sid = "a" * 32
    apikey = "api-key"
    token = "b" * 32
    source = "LM-EXOTEL"
    targets = ("6" * 11, "7" * 11)

    response = requests.Request()
    response.status_code = requests.codes.ok

    mock_post.return_value = response

    with pytest.raises(TypeError):
        NotifyExotel(sid=sid, token=token, source=" ")

    with pytest.raises(TypeError):
        NotifyExotel(sid=sid, token=token, source=source, apikey=" ")

    with pytest.raises(TypeError):
        NotifyExotel(sid=sid, token=token, source=source, region_name=" ")

    with pytest.raises(TypeError):
        NotifyExotel(sid=sid, token=token, source=source, priority=" ")

    # Programmatic priority shorthand is supported as an Apprise convenience.
    obj = NotifyExotel(sid=sid, token=token, source="5" * 11, priority="+")
    assert obj.priority == "high"
    assert obj.targets == ["5" * 11]

    mock_post.reset_mock()
    assert obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
    assert mock_post.call_count == 1
    payload = mock_post.call_args_list[0][1]["data"]
    assert payload["From"] == "5" * 11
    assert payload["To"] == "5" * 11
    assert payload["Priority"] == "high"

    # When a Sender ID is provided without targets, pass it upstream as the
    # implicit recipient and let Exotel decide whether it is deliverable.
    obj = NotifyExotel(sid=sid, token=token, source=source)
    assert obj.targets == [source]
    assert len(obj) == 1

    mock_post.reset_mock()
    assert obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
    assert mock_post.call_count == 1
    payload = mock_post.call_args_list[0][1]["data"]
    assert payload["From"] == source
    assert payload["To"] == source

    obj = Apprise.instantiate(
        "exotel://{}:{}@{}/{}?apikey={}&region=in&unicode=no"
        "&priority=high".format(sid, token, source, "/".join(targets), apikey)
    )

    assert isinstance(obj, NotifyExotel)
    assert len(obj) == len(targets)
    assert set(obj.targets) == set(targets)

    mock_post.reset_mock()
    assert obj.notify(body="body", title="title", notify_type=NotifyType.INFO)

    assert mock_post.call_count == len(targets)

    first_call = mock_post.call_args_list[0]
    assert first_call[0][0] == (
        "https://api.in.exotel.com/v1/Accounts/{}/Sms/send".format(sid)
    )
    assert first_call[1]["auth"] == (apikey, token)

    first_payload = first_call[1]["data"]
    assert first_payload["From"] == source
    assert first_payload["To"] == obj.targets[0]
    assert first_payload["Body"] == "title\r\nbody"
    assert first_payload["EncodingType"] == "plain"
    assert first_payload["Priority"] == "high"
    assert "StatusCallback" not in first_payload

    second_call = mock_post.call_args_list[1]
    assert second_call[0][0] == (
        "https://api.in.exotel.com/v1/Accounts/{}/Sms/send".format(sid)
    )
    assert second_call[1]["auth"] == (apikey, token)

    second_payload = second_call[1]["data"]
    assert second_payload["From"] == source
    assert second_payload["To"] == obj.targets[1]
    assert second_payload["Body"] == "title\r\nbody"
    assert second_payload["EncodingType"] == "plain"
    assert second_payload["Priority"] == "high"
    assert "StatusCallback" not in second_payload

    obj = Apprise.instantiate(
        "exotel://{}:{}@{}/{}?apikey={}&region=in&unicode=no"
        "&priority=high&batch=yes".format(
            sid, token, source, "/".join(targets), apikey
        )
    )
    assert isinstance(obj, NotifyExotel)
    assert len(obj) == 1
    assert set(obj.targets) == set(targets)

    mock_post.reset_mock()
    assert obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
    assert mock_post.call_count == 1

    first_call = mock_post.call_args_list[0]
    assert first_call[0][0] == (
        "https://api.in.exotel.com/v1/Accounts/{}/Sms/send".format(sid)
    )
    assert first_call[1]["auth"] == (apikey, token)

    payload = first_call[1]["data"]
    assert payload["From"] == source
    assert payload["To"] == obj.targets
    assert payload["Body"] == "title\r\nbody"
    assert payload["EncodingType"] == "plain"
    assert payload["Priority"] == "high"
    assert "StatusCallback" not in payload
    assert "batch=yes" in obj.url()

    batch_targets = tuple("6{:010d}".format(i) for i in range(101))
    batch_obj = NotifyExotel(
        sid=sid,
        token=token,
        source=source,
        targets=batch_targets,
        batch=True,
    )
    assert len(batch_obj) == 2

    mock_post.reset_mock()
    assert batch_obj.notify(
        body="body", title="title", notify_type=NotifyType.INFO
    )
    assert mock_post.call_count == 2

    first_payload = mock_post.call_args_list[0][1]["data"]
    assert first_payload["To"] == list(batch_targets[:100])

    second_payload = mock_post.call_args_list[1][1]["data"]
    assert second_payload["To"] == list(batch_targets[100:])

    # Targets must not be part of the URL identifier.
    obj_cmp = Apprise.instantiate(
        "exotel://{}:{}@{}/{}?region=in".format(sid, token, source, "8" * 11)
    )
    assert obj.url_id() != obj_cmp.url_id()

    obj_cmp = Apprise.instantiate(
        "exotel://{}:{}@{}/{}?apikey={}&region=in".format(
            sid, token, source, "8" * 11, apikey
        )
    )
    assert obj.url_id() == obj_cmp.url_id()

    obj_cmp = Apprise.instantiate(
        "exotel://{}:{}@{}/{}?key={}&region=in".format(
            sid, token, source, "8" * 11, apikey
        )
    )
    assert obj.url_id() == obj_cmp.url_id()

    # Region changes the upstream API host and should alter the identifier.
    obj_cmp = Apprise.instantiate(
        "exotel://{}:{}@{}/{}?apikey={}&region=us".format(
            sid, token, source, "8" * 11, apikey
        )
    )
    assert obj.url_id() != obj_cmp.url_id()

    # Even if all provided targets are invalid, Apprise treats the notifier
    # as one logical request target for length purposes.
    obj = Apprise.instantiate(f"exotel://{sid}:{token}@{source}/abcd")
    assert isinstance(obj, NotifyExotel)
    assert len(obj) == 1



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from json import dumps, loads

# Disable logging for a cleaner testing output
import logging
import os
from unittest import mock

from helpers import AppriseURLTester
import requests

from apprise import Apprise, NotifyType
from apprise.plugins.notificationapi import NotifyNotificationAPI

logging.disable(logging.CRITICAL)

NOTIFICATIONAPI_GOOD_RESPONSE = dumps({})

NOTIFICATIONAPI_BAD_RESPONSE = "{"

# Attachment Directory
TEST_VAR_DIR = os.path.join(os.path.dirname(__file__), "var")

# Our Testing URLs
apprise_url_tests = (
    (
        "napi://",
        {
            "instance": TypeError,
        },
    ),
    (
        "napi://:@/",
        {
            "instance": TypeError,
        },
    ),
    (
        "napi://abcd",
        {
            # invalid from email
            "instance": TypeError,
        },
    ),
    (
        "napi://abcd@host.com",
        {
            # Just an Email specified, no client_id or client_secret
            "instance": TypeError,
        },
    ),
    (
        "napi://user@client_id/cs14a/user@example.ca",
        {
            # No id matched
            "instance": TypeError,
        },
    ),
    (
        "napi://user@client_id/cs14b/+15551235553",
        {
            # No id matched
            "instance": TypeError,
        },
    ),
    (
        "napi://user@client_id/cs14c/+15551235553/user@example.ca",
        {
            # No id matched
            "instance": TypeError,
        },
    ),
    (
        "napi://type@client_id/client_secret/id/+15551235553/?mode=invalid",
        {
            # Invalid mode
            "instance": TypeError,
        },
    ),
    (
        "napi://type@client_id/client_secret/id/+15551235553/?region=invalid",
        {
            # Invalid region
            "instance": TypeError,
        },
    ),
    (
        (
            "napi://type@client_id/client_secret/id/user@example.ca/"
            "user2@example.ca"
        ),
        {
            # to many emails assigned to id (variation 1)
            "instance": TypeError,
        },
    ),
    (
        (
            "napi://type@client_id/client_secret/user@example.ca/"
            "user2@example.ca"
        ),
        {
            # to many emails assigned to id (variation 2)
            "instance": TypeError,
        },
    ),
    (
        ("napi://type@client_id/client_secret/id/+15551235553/+15551235555"),
        {
            # to many phone no's assigned to id (variation 1)
            "instance": TypeError,
        },
    ),
    (
        ("napi://type@client_id/client_secret/+15551235553/+15551235555"),
        {
            # to many phone no's assigned to id (variation 2)
            "instance": TypeError,
        },
    ),
    (
        "napi://type@client_id/client_secret/id/+15551235553/?mode=invalid",
        {
            # Invalid mode
            "instance": TypeError,
        },
    ),
    (
        "napi://client_id/client_secret/id/+15551231234/?type=*(",
        {
            # Invalid type
            "instance": TypeError,
        },
    ),
    (
        "napi://client_id/client_secret/id/+15551231234/?channels=bad",
        {
            # Invalid channel
            "instance": TypeError,
        },
    ),
    (
        "napi://?secret=cs&to=id,user404@example.com&type=typed",
        {
            # No id found
            "instance": TypeError,
        },
    ),
    (
        "napi://client_id/client_secret/id/g@rb@ge/+15551235553/",
        {
            # g@rb@ge entry ignored
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://cid/secret/id/user1@example.com/?type=apprise-msg",
        {
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "notificationapi://cid/secret/id/user1@example.com",
        {
            # Support full schema:// of notificationapi://
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://cid/secret/id/id2/user1@example.com",
        {
            # two id's in a row
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        (
            "napi://type@cid/secret/id10/user2@example.com/"
            "id5/+15551235555/id8/+15551235534"
            "?reply=Chris<chris@example.com>"
        ),
        {
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        (
            "napi://type@cid/secret/abc1/user1@example.com/"
            "id5/+15551235555/?from=Chris&reply=Christopher"
        ),
        {
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        (
            "napi://type@cid/secret/id/user3@example.com/"
            "?from=joe@example.ca&reply=user@abc.com"
        ),
        {
            # Set from/source
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        (
            "napi://type@cid/secret/id/user4@example.com/"
            "?from=joe@example.ca&bcc=user1@yahoo.ca&cc=user2@yahoo.ca"
        ),
        {
            # Set from/source
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "napi://type@c...d/s...t/",
        },
    ),
    (
        "napi://?id=ci&secret=cs&to=id,user5@example.com&type=typec",
        {
            # use just kwargs
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "napi://typec@c...i/c...s/",
        },
    ),
    (
        "napi://id?secret=cs&to=id,user5@example.com&type=typeb",
        {
            # id is pull from the host
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "napi://typeb@i...d/c...s/",
        },
    ),
    (
        "napi://secret?id=ci&to=id,user5@example.com&type=typea",
        {
            # id pulled from kwargs still allows secret to be the
            # next parsed entry from cli
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "napi://typea@c...i/s...t/",
        },
    ),
    (
        "napi://?id=ci&secret=cs&type=test-type&region=eu",
        {
            # No targets specified
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
            "notify_response": False,
        },
    ),
    (
        "napi://?id=ci&secret=cs&to=id,user5@example.com&type=typec",
        {
            # bad response
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_BAD_RESPONSE,
            "notify_response": False,
        },
    ),
    (
        "napi://user@client_id/cs2/id/user6@example.ca?bcc=invalid",
        {
            # A good email with a bad Blind Carbon Copy
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs3/id/user8@example.ca?cc=l2g@nuxref.com",
        {
            # A good email with Carbon Copy
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://client_id/cs3/id/user8@example.ca"
        "?channels=email,sms,slack,mobile_push,web_push,inapp",
        {
            # A good email with Carbon Copy
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs4/id/user9@example.ca"
        "?cc=Chris<l2g@nuxref.com>",
        {
            # A good email with Carbon Copy
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs5/id/user10@example.ca?cc=invalid",
        {
            # A good email with Carbon Copy
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs6/id/user11@example.ca?to=invalid",
        {
            # an invalid to email
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs7/id/chris1@example.com",
        {
            # An email with a designated to email
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs8/id1/user12@example.ca"
        "?to=id,Chris<chris2@example.com>",
        {
            # An email with a full name in in To field
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs9/id2/user13@example.ca/"
        "id/kris@example.com/id/chris2@example.com/id/+15552341234"
        "?:token=value",
        {
            # Several emails to notify
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs10/id/user14@example.ca"
        "?cc=Chris<chris10@example.com>",
        {
            # An email with a full name in cc
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs11/id/user15@example.ca"
        "?cc=chris12@example.com",
        {
            # An email with a full name in cc
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs12/id/user16@example.ca"
        "?bcc=Chris<chris14@example.com>",
        {
            # An email with a full name in bcc
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs13/id/user@example.ca"
        "?bcc=chris13@example.com",
        {
            # An email with a full name in bcc
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs14/id/user@example.ca"
        "?to=Chris<chris9@example.com>,id14",
        {
            # An email with a full name in bcc
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs15/id?to=user@example.com",
        {
            # An email with a full name in bcc
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs16/id/user@example.ca"
        "?template=1234&+sub=value&+sub2=value2",
        {
            # A good email with a template + substitutions
            "instance": NotifyNotificationAPI,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "napi://user@c...d/c...6/",
        },
    ),
    (
        "napi://user@client_id/cs17/id/user@example.ca",
        {
            "instance": NotifyNotificationAPI,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs18/id/user@example.ca",
        {
            "instance": NotifyNotificationAPI,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
    (
        "napi://user@client_id/cs19/id/user@example.ca",
        {
            "instance": NotifyNotificationAPI,
            # Throws connection and transfer exceptions when this
            # flag is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
            "requests_response_text": NOTIFICATIONAPI_GOOD_RESPONSE,
        },
    ),
)


def test_plugin_napi_urls():
    """
    NotifyNotificationAPI() Apprise URLs

    """

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_napi_template_sms_payloads(mock_post):
    """NotifyNotificationAPI() Testing Template SMS Payloads."""

    okay_response = requests.Request()
    okay_response.status_code = requests.codes.ok
    okay_response.content = NOTIFICATIONAPI_GOOD_RESPONSE

    # Assign our mock object our return value
    mock_post.return_value = okay_response

    # Details
    client_id = "my_id"
    client_secret = "my_secret"
    message_type = "apprise-post"
    targets = "userid/+1-555-123-4567"

    obj = Apprise.instantiate(
        f"napi://{message_type}@{client_id}/{client_secret}/"
        f"{targets}?mode=template"
    )
    assert isinstance(obj, NotifyNotificationAPI)
    assert isinstance(obj.url(), str)

    # No calls made yet
    assert mock_post.call_count == 0

    # Send our notification
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )

    # delivery of message
    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == f"https://api.notificationapi.com/{client_id}/sender"
    )

    payload = loads(mock_post.call_args_list[0][1]["data"])
    assert payload == {
        "type": "apprise-post",
        "to": {
            "id": "userid",
            "number": "+15551234567",
        },
        "parameters": {
            "appBody": "body",
            "appTitle": "title",
            "appType": "info",
            "appId": "Apprise",
            "appDescription": "Apprise Notifications",
            "appColor": "#3AA3E3",
            "appImageUrl": (
                "https://github.com/caronc/apprise/raw/master/apprise"
                "/assets/themes/default/apprise-info-72x72.png"
            ),
            "appUrl": "https://github.com/caronc/apprise",
        },
    }
    headers = mock_post.call_args_list[0][1]["headers"]
    assert headers == {
        "User-Agent": "Apprise",
        "Content-Type": "application/json",
        "Authorization": "Basic bXlfaWQ6bXlfc2VjcmV0",
    }

    # Reset our mock object
    mock_post.reset_mock()


@mock.patch("requests.post")
def test_plugin_napi_template_email_payloads(mock_post):
    """NotifyNotificationAPI() Testing Template Email Payloads."""

    okay_response = requests.Request()
    okay_response.status_code = requests.codes.ok
    okay_response.content = NOTIFICATIONAPI_GOOD_RESPONSE

    # Assign our mock object our return value
    mock_post.return_value = okay_response

    # Details
    client_id = "my_id_abc"
    client_secret = "my_secret"
    message_type = "apprise-post"
    targets = "userid/test@example.ca"

    obj = Apprise.instantiate(
        f"napi://{message_type}@{client_id}/{client_secret}/"
        f"{targets}?from=Chris<chris@example.eu>&bcc=joe@hidden.com&"
        f"cc=jason@hidden.com&:customToken=customValue&mode=template"
    )
    assert isinstance(obj, NotifyNotificationAPI)
    assert isinstance(obj.url(), str)

    # No calls made yet
    assert mock_post.call_count == 0

    # Send our notification
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )

    # delivery of message
    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == f"https://api.notificationapi.com/{client_id}/sender"
    )

    payload = loads(mock_post.call_args_list[0][1]["data"])
    assert payload == {
        "type": "apprise-post",
        "to": {
            "id": "userid",
            "email": "test@example.ca",
        },
        "options": {
            "email": {
                "fromAddress": "chris@example.eu",
                "fromName": "Chris",
                "ccAddresses": ["jason@hidden.com"],
                "bccAddresses": ["joe@hidden.com"],
            }
        },
        "parameters": {
            "customToken": "customValue",
            "appBody": "body",
            "appTitle": "title",
            "appType": "info",
            "appId": "Apprise",
            "appDescription": "Apprise Notifications",
            "appColor": "#3AA3E3",
            "appImageUrl": (
                "https://github.com/caronc/apprise/raw/master/apprise/"
                "assets/themes/default/apprise-info-72x72.png"
            ),
            "appUrl": "https://github.com/caronc/apprise",
        },
    }
    headers = mock_post.call_args_list[0][1]["headers"]
    assert headers == {
        "User-Agent": "Apprise",
        "Content-Type": "application/json",
        "Authorization": "Basic bXlfaWRfYWJjOm15X3NlY3JldA==",
    }

    # Reset our mock object
    mock_post.reset_mock()


@mock.patch("requests.post")
def test_plugin_napi_message_payloads(mock_post):
    """NotifyNotificationAPI() Testing Message Payloads."""

    okay_response = requests.Request()
    okay_response.status_code = requests.codes.ok
    okay_response.content = NOTIFICATIONAPI_GOOD_RESPONSE

    # Assign our mock object our return value
    mock_post.return_value = okay_response

    # Details
    client_id = "my_id_abc"
    client_secret = "my_secret"
    message_type = "apprise-post"
    targets = "userid/test@example.ca/+15551239876"

    obj = Apprise.instantiate(
        f"napi://{message_type}@{client_id}/{client_secret}/"
        f"{targets}?from=Chris<chris@example.eu>&bcc=joe@hidden.com"
        f"&mode=message"
    )
    assert isinstance(obj, NotifyNotificationAPI)
    assert isinstance(obj.url(), str)

    # No calls made yet
    assert mock_post.call_count == 0

    # Send our notification
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )

    # delivery of message
    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == f"https://api.notificationapi.com/{client_id}/sender"
    )

    payload = loads(mock_post.call_args_list[0][1]["data"])
    assert payload == {
        "type": "apprise-post",
        "to": {
            "id": "userid",
            "email": "test@example.ca",
            "number": "+15551239876",
        },
        "email": {
            "subject": "title",
            "html": "body",
            "senderName": "Chris",
            "senderEmail": "chris@example.eu",
        },
        "options": {
            "email": {
                "fromAddress": "chris@example.eu",
                "fromName": "Chris",
                "bccAddresses": ["joe@hidden.com"],
            },
        },
    }
    headers = mock_post.call_args_list[0][1]["headers"]
    assert headers == {
        "User-Agent": "Apprise",
        "Content-Type": "application/json",
        "Authorization": "Basic bXlfaWRfYWJjOm15X3NlY3JldA==",
    }

    # Reset our mock object
    mock_post.reset_mock()

    # Reversing the sms with email causes auto-detection channel to
    # be sms instead of email
    targets = "userid/+15551239876/test@example.ca"

    obj = Apprise.instantiate(
        f"napi://{client_id}/{client_secret}/"
        f"{targets}?from=Chris<chris@example.eu>&bcc=joe@hidden.com"
    )
    assert isinstance(obj, NotifyNotificationAPI)
    assert isinstance(obj.url(), str)

    # No calls made yet
    assert mock_post.call_count == 0

    # Send our notification
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )

    # delivery of message
    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == f"https://api.notificationapi.com/{client_id}/sender"
    )

    payload = loads(mock_post.call_args_list[0][1]["data"])
    assert payload == {
        "type": "apprise",
        "to": {
            "id": "userid",
            "number": "+15551239876",
            "email": "test@example.ca",
        },
        "sms": {"message": "title\nbody"},
        "options": {
            "email": {
                "fromAddress": "chris@example.eu",
                "fromName": "Chris",
                "bccAddresses": ["joe@hidden.com"],
            },
        },
    }

    headers = mock_post.call_args_list[0][1]["headers"]
    assert headers == {
        "User-Agent": "Apprise",
        "Content-Type": "application/json",
        "Authorization": "Basic bXlfaWRfYWJjOm15X3NlY3JldA==",
    }

    # Reset our mock object
    mock_post.reset_mock()

    # Experiment with fixed channels:
    obj = Apprise.instantiate(
        f"napi://{message_type}@{client_id}/{client_secret}/"
        f"{targets}?from=Chris<chris@example.eu>&bcc=joe@hidden.com"
        f"&mode=message&channels=sms,slack"
    )
    assert isinstance(obj, NotifyNotificationAPI)
    assert isinstance(obj.url(), str)

    # No calls made yet
    assert mock_post.call_count == 0

    # Send our notification
    assert (
        obj.notify(body="body", title="title", notify_type=NotifyType.INFO)
        is True
    )

    # delivery of message
    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == f"https://api.notificationapi.com/{client_id}/sender"
    )

    payload = loads(mock_post.call_args_list[0][1]["data"])
    assert payload == {
        "type": "apprise-post",
        "to": {
            "id": "userid",
            "email": "test@example.ca",
            "number": "+15551239876",
        },
        "slack": {"text": "title\nbody"},
        "sms": {"message": "title\nbody"},
        "options": {
            "email": {
                "fromAddress": "chris@example.eu",
                "fromName": "Chris",
                "bccAddresses": ["joe@hidden.com"],
            },
        },
    }

    headers = mock_post.call_args_list[0][1]["headers"]
    assert headers == {
        "User-Agent": "Apprise",
        "Content-Type": "application/json",
        "Authorization": "Basic bXlfaWRfYWJjOm15X3NlY3JldA==",
    }


def test_plugin_napi_edge_cases():
    """
    NotifyNotificationAPI() Edge Cases

    """
    client_id = "my_id_abc"
    client_secret = "my_secret"
    targets = ["userid", "test@example.ca", "+15551239876"]

    # Tests case where tokens is == None
    obj = NotifyNotificationAPI(client_id, client_secret, targets=targets)
    assert isinstance(obj, NotifyNotificationAPI)
    assert isinstance(obj.url(), str)



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from json import dumps

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise.plugins.vonage import NotifyVonage

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "vonage://",
        {
            # No API Key specified
            "instance": TypeError,
        },
    ),
    (
        "vonage://:@/",
        {
            # invalid Auth key
            "instance": TypeError,
        },
    ),
    (
        "vonage://AC{}@12345678".format("a" * 8),
        {
            # Just a key provided
            "instance": TypeError,
        },
    ),
    (
        "vonage://AC{}:{}@{}".format("a" * 8, "b" * 16, "3" * 9),
        {
            # key and secret provided and from but invalid from no
            "instance": TypeError,
        },
    ),
    (
        "vonage://AC{}:{}@{}/?ttl=0".format("b" * 8, "c" * 16, "3" * 11),
        {
            # Invalid ttl defined
            "instance": TypeError,
        },
    ),
    (
        "vonage://AC{}:{}@{}".format("d" * 8, "e" * 16, "a" * 11),
        {
            # Invalid source number
            "instance": TypeError,
        },
    ),
    (
        "vonage://AC{}:{}@{}/123/{}/abcd/".format(
            "f" * 8, "g" * 16, "3" * 11, "9" * 15
        ),
        {
            # valid everything but target numbers
            "instance": NotifyVonage,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "vonage://A...f:****@",
        },
    ),
    (
        "vonage://AC{}:{}@{}".format("h" * 8, "i" * 16, "5" * 11),
        {
            # using phone no with no target - we text ourselves in
            # this case
            "instance": NotifyVonage,
        },
    ),
    (
        "vonage://_?key=AC{}&secret={}&from={}".format(
            "a" * 8, "b" * 16, "5" * 11
        ),
        {
            # use get args to acomplish the same thing
            "instance": NotifyVonage,
        },
    ),
    (
        "vonage://_?key=AC{}&secret={}&source={}".format(
            "a" * 8, "b" * 16, "5" * 11
        ),
        {
            # use get args to acomplish the same thing (use source instead
            # of from)
            "instance": NotifyVonage,
        },
    ),
    (
        "vonage://_?key=AC{}&secret={}&from={}&to={}".format(
            "a" * 8, "b" * 16, "5" * 11, "7" * 13
        ),
        {
            # use to=
            "instance": NotifyVonage,
        },
    ),
    (
        "vonage://AC{}:{}@{}".format("a" * 8, "b" * 16, "6" * 11),
        {
            "instance": NotifyVonage,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "vonage://AC{}:{}@{}".format("a" * 8, "b" * 16, "6" * 11),
        {
            "instance": NotifyVonage,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
    # Nexmo Backwards Support
    (
        "nexmo://",
        {
            # No API Key specified
            "instance": TypeError,
        },
    ),
    (
        "nexmo://:@/",
        {
            # invalid Auth key
            "instance": TypeError,
        },
    ),
    (
        "nexmo://AC{}@12345678".format("a" * 8),
        {
            # Just a key provided
            "instance": TypeError,
        },
    ),
    (
        "nexmo://AC{}:{}@{}".format("a" * 8, "b" * 16, "3" * 9),
        {
            # key and secret provided and from but invalid from no
            "instance": TypeError,
        },
    ),
    (
        "nexmo://AC{}:{}@{}/?ttl=0".format("b" * 8, "c" * 16, "3" * 11),
        {
            # Invalid ttl defined
            "instance": TypeError,
        },
    ),
    (
        "nexmo://AC{}:{}@{}".format("d" * 8, "e" * 16, "a" * 11),
        {
            # Invalid source number
            "instance": TypeError,
        },
    ),
    (
        "nexmo://AC{}:{}@{}/123/{}/abcd/".format(
            "f" * 8, "g" * 16, "3" * 11, "9" * 15
        ),
        {
            # valid everything but target numbers
            "instance": NotifyVonage,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "vonage://A...f:****@",
        },
    ),
    (
        "nexmo://AC{}:{}@{}".format("h" * 8, "i" * 16, "5" * 11),
        {
            # using phone no with no target - we text ourselves in
            # this case
            "instance": NotifyVonage,
        },
    ),
    (
        "nexmo://_?key=AC{}&secret={}&from={}".format(
            "a" * 8, "b" * 16, "5" * 11
        ),
        {
            # use get args to acomplish the same thing
            "instance": NotifyVonage,
        },
    ),
    (
        "nexmo://_?key=AC{}&secret={}&source={}".format(
            "a" * 8, "b" * 16, "5" * 11
        ),
        {
            # use get args to acomplish the same thing (use source instead of
            # from)
            "instance": NotifyVonage,
        },
    ),
    (
        "nexmo://_?key=AC{}&secret={}&from={}&to={}".format(
            "a" * 8, "b" * 16, "5" * 11, "7" * 13
        ),
        {
            # use to=
            "instance": NotifyVonage,
        },
    ),
    (
        "nexmo://AC{}:{}@{}".format("a" * 8, "b" * 16, "6" * 11),
        {
            "instance": NotifyVonage,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "nexmo://AC{}:{}@{}".format("a" * 8, "b" * 16, "6" * 11),
        {
            "instance": NotifyVonage,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_vonage_urls():
    """NotifyVonage() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_vonage_edge_cases(mock_post):
    """NotifyVonage() Edge Cases."""

    # Prepare our response
    response = requests.Request()
    response.status_code = requests.codes.ok

    # Prepare Mock
    mock_post.return_value = response

    # Initialize some generic (but valid) tokens
    apikey = "AC{}".format("b" * 8)
    secret = "{}".format("b" * 16)
    source = "+1 (555) 123-3456"

    # No apikey specified
    with pytest.raises(TypeError):
        NotifyVonage(apikey=None, secret=secret, source=source)

    with pytest.raises(TypeError):
        NotifyVonage(apikey="  ", secret=secret, source=source)

    # No secret specified
    with pytest.raises(TypeError):
        NotifyVonage(apikey=apikey, secret=None, source=source)

    with pytest.raises(TypeError):
        NotifyVonage(apikey=apikey, secret="  ", source=source)

    # a error response
    response.status_code = 400
    response.content = dumps(
        {
            "code": 21211,
            "message": "The 'To' number +1234567 is not a valid phone number.",
        }
    )
    mock_post.return_value = response

    # Initialize our object
    obj = NotifyVonage(apikey=apikey, secret=secret, source=source)

    # We will fail with the above error code
    assert obj.notify("title", "body", "info") is False



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from json import loads

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise import Apprise, NotifyType
from apprise.plugins.msg91 import NotifyMSG91

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "msg91://",
        {
            # No hostname/authkey specified
            "instance": TypeError,
        },
    ),
    (
        "msg91://-",
        {
            # Invalid AuthKey
            "instance": TypeError,
        },
    ),
    (
        "msg91://{}".format("a" * 23),
        {
            # valid AuthKey but no Template ID
            "instance": TypeError,
        },
    ),
    (
        "msg91://{}@{}".format("t" * 20, "a" * 23),
        {
            # Valid entry but no targets
            "instance": NotifyMSG91,
            # Since there are no targets specified we expect a False return on
            # send()
            "notify_response": False,
        },
    ),
    (
        "msg91://{}@{}/abcd".format("t" * 20, "a" * 23),
        {
            # No number to notify
            "instance": NotifyMSG91,
            # Since there are no targets specified we expect a False return on
            # send()
            "notify_response": False,
        },
    ),
    (
        "msg91://{}@{}/15551232000".format("t" * 20, "a" * 23),
        {
            # a valid message
            "instance": NotifyMSG91,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "msg91://t...t@a...a/15551232000",
        },
    ),
    (
        "msg91://{}@{}/?to=15551232000&short_url=no".format(
            "t" * 20, "a" * 23
        ),
        {
            # a valid message
            "instance": NotifyMSG91,
        },
    ),
    (
        "msg91://{}@{}/15551232000?short_url=yes".format("t" * 20, "a" * 23),
        {
            # testing short_url
            "instance": NotifyMSG91,
        },
    ),
    (
        "msg91://{}@{}/15551232000".format("t" * 20, "a" * 23),
        {
            # use get args to acomplish the same thing
            "instance": NotifyMSG91,
        },
    ),
    (
        "msg91://{}@{}/15551232000".format("t" * 20, "a" * 23),
        {
            "instance": NotifyMSG91,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "msg91://{}@{}/15551232000".format("t" * 20, "a" * 23),
        {
            "instance": NotifyMSG91,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_msg91_urls():
    """NotifyMSG91() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_msg91_edge_cases(mock_post):
    """NotifyMSG91() Edge Cases."""

    # Prepare our response
    response = requests.Request()
    response.status_code = requests.codes.ok

    # Prepare Mock
    mock_post.return_value = response

    # Initialize some generic (but valid) tokens
    target = "+1 (555) 123-3456"

    # No authkey specified
    with pytest.raises(TypeError):
        NotifyMSG91(template="1234", authkey=None, targets=target)
    with pytest.raises(TypeError):
        NotifyMSG91(template="1234", authkey="    ", targets=target)
    with pytest.raises(TypeError):
        NotifyMSG91(template="     ", authkey="a" * 23, targets=target)
    with pytest.raises(TypeError):
        NotifyMSG91(template=None, authkey="a" * 23, targets=target)


@mock.patch("requests.post")
def test_plugin_msg91_keywords(mock_post):
    """NotifyMSG91() Templating."""

    response = mock.Mock()
    response.content = ""
    response.status_code = requests.codes.ok

    # Prepare Mock
    mock_post.return_value = response

    target = "+1 (555) 123-3456"
    template = "12345"
    authkey = "{}".format("b" * 32)

    message_contents = "test"

    # Variation of initialization without API key
    obj = Apprise.instantiate(
        f"msg91://{template}@{authkey}/{target}?:key=value&:mobiles=ignored"
    )
    assert isinstance(obj, NotifyMSG91)
    assert isinstance(obj.url(), str)

    # Send Notification
    assert obj.send(body=message_contents) is True

    # Validate expected call parameters
    assert mock_post.call_count == 1
    first_call = mock_post.call_args_list[0]

    # URL and message parameters are the same for both calls
    assert first_call[0][0] == "https://control.msg91.com/api/v5/flow/"
    response = loads(first_call[1]["data"])
    assert response["template_id"] == template
    assert response["short_url"] == 0
    assert len(response["recipients"]) == 1
    # mobiles is not over-ridden as it is a special reserved token
    assert response["recipients"][0]["mobiles"] == "15551233456"

    # Our base tokens
    assert response["recipients"][0]["body"] == message_contents
    assert response["recipients"][0]["type"] == NotifyType.INFO.value
    assert response["recipients"][0]["key"] == "value"

    mock_post.reset_mock()

    # Play with mapping
    obj = Apprise.instantiate(
        f"msg91://{template}@{authkey}/{target}?:body&:type=cat"
    )
    assert isinstance(obj, NotifyMSG91)
    assert isinstance(obj.url(), str)

    # Send Notification
    assert obj.send(body=message_contents) is True

    # Validate expected call parameters
    assert mock_post.call_count == 1
    first_call = mock_post.call_args_list[0]
    assert "NotifyType." not in first_call[1]["data"]

    # URL and message parameters are the same for both calls
    assert first_call[0][0] == "https://control.msg91.com/api/v5/flow/"
    response = loads(first_call[1]["data"])
    assert response["template_id"] == template
    assert response["short_url"] == 0
    assert len(response["recipients"]) == 1
    assert response["recipients"][0]["mobiles"] == "15551233456"
    assert "body" not in response["recipients"][0]
    assert "type" not in response["recipients"][0]
    assert response["recipients"][0]["cat"] == "info"



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from json import dumps

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import pytest
import requests

from apprise.plugins.burstsms import NotifyBurstSMS

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "burstsms://",
        {
            # No API Key specified
            "instance": TypeError,
        },
    ),
    (
        "burstsms://:@/",
        {
            # invalid Auth key
            "instance": TypeError,
        },
    ),
    (
        "burstsms://{}@12345678".format("a" * 8),
        {
            # Just a key provided
            "instance": TypeError,
        },
    ),
    (
        "burstsms://{}:{}@%20".format("d" * 8, "e" * 16),
        {
            # Invalid source number
            "instance": TypeError,
        },
    ),
    (
        "burstsms://{}:{}@{}/123/{}/abcd/".format(
            "f" * 8, "g" * 16, "3" * 11, "9" * 15
        ),
        {
            # valid everything but target numbers
            "instance": NotifyBurstSMS,
            # Expected notify() response because not all targets are valid
            "notify_response": False,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "burstsms://f...f:****@",
        },
    ),
    (
        "burstsms://{}:{}@{}".format("h" * 8, "i" * 16, "5" * 11),
        {
            "instance": NotifyBurstSMS,
            # Expected notify() response because no targets are defined
            "notify_response": False,
        },
    ),
    (
        "burstsms://_?key={}&secret={}&from={}&to={}".format(
            "a" * 8, "b" * 16, "5" * 11, "6" * 11
        ),
        {
            # use get args to acomplish the same thing
            "instance": NotifyBurstSMS,
        },
    ),
    (
        "burstsms://_?key={}&secret={}&from={}&to={}&batch=y".format(
            "a" * 8, "b" * 16, "5" * 11, "6" * 11
        ),
        {
            # batch flag set
            "instance": NotifyBurstSMS,
        },
    ),
    # Test our country
    (
        "burstsms://_?key={}&secret={}&source={}&to={}&country=us".format(
            "a" * 8, "b" * 16, "5" * 11, "6" * 11
        ),
        {
            "instance": NotifyBurstSMS,
        },
    ),
    # Test an invalid country
    (
        "burstsms://_?key={}&secret={}&source={}&to={}&country=invalid".format(
            "a" * 8, "b" * 16, "5" * 11, "6" * 11
        ),
        {
            "instance": TypeError,
        },
    ),
    # Test our validity
    (
        "burstsms://_?key={}&secret={}&source={}&to={}&validity=10".format(
            "a" * 8, "b" * 16, "5" * 11, "6" * 11
        ),
        {
            "instance": NotifyBurstSMS,
        },
    ),
    # Test an invalid country
    (
        "burstsms://_?key={}&secret={}&source={}&to={}&validity=invalid".format(
            "a" * 8, "b" * 16, "5" * 11, "6" * 11
        ),
        {
            "instance": TypeError,
        },
    ),
    (
        "burstsms://_?key={}&secret={}&from={}&to={}".format(
            "a" * 8, "b" * 16, "5" * 11, "7" * 11
        ),
        {
            # use to=
            "instance": NotifyBurstSMS,
        },
    ),
    (
        "burstsms://{}:{}@{}/{}".format("a" * 8, "b" * 16, "6" * 11, "7" * 11),
        {
            "instance": NotifyBurstSMS,
            # throw a bizarre code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "burstsms://{}:{}@{}/{}".format("a" * 8, "b" * 16, "6" * 11, "7" * 11),
        {
            "instance": NotifyBurstSMS,
            # Throws a series of i/o exceptions with this flag
            # is set and tests that we gracefully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_burstsms_urls():
    """NotifyBurstSMS() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_burstsms_edge_cases(mock_post):
    """NotifyBurstSMS() Edge Cases."""

    # Prepare our response
    response = requests.Request()
    response.status_code = requests.codes.ok

    # Prepare Mock
    mock_post.return_value = response

    # Initialize some generic (but valid) tokens
    apikey = "{}".format("b" * 8)
    secret = "{}".format("b" * 16)
    source = "+1 (555) 123-3456"

    # No apikey specified
    with pytest.raises(TypeError):
        NotifyBurstSMS(apikey=None, secret=secret, source=source)

    with pytest.raises(TypeError):
        NotifyBurstSMS(apikey="  ", secret=secret, source=source)

    # No secret specified
    with pytest.raises(TypeError):
        NotifyBurstSMS(apikey=apikey, secret=None, source=source)

    with pytest.raises(TypeError):
        NotifyBurstSMS(apikey=apikey, secret="  ", source=source)

    # a error response
    response.status_code = 400
    response.content = dumps(
        {
            "error": {
                "code": "FIELD_INVALID",
                "description": (
                    "Sender ID must be one of the numbers that are currently"
                    " leased."
                ),
            },
        }
    )
    mock_post.return_value = response

    # Initialize our object
    obj = NotifyBurstSMS(apikey=apikey, secret=secret, source=source)

    # We will fail with the above error code
    assert obj.notify("title", "body", "info") is False



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""IRC Notifications.

This is simplified IRC client designed for notification delivery.
It focuses on reliability and predictable behaviour, not full IRC features.

URL formats (examples):
  - irc://hostname/#channel/@user
  - irc://user@hostname/#channel
  - irc://user:password@hostname/#channel
  - ircs://hostname/#channel  (TLS, default port 6697)

Targets:
  - Channels are specified as #channel
  - Users are specified as @nickname
"""

from __future__ import annotations

from itertools import chain
import re
from typing import Any, Optional

from ...common import NotifyType
from ...locale import gettext_lazy as _
from ...url import PrivacyMode
from ...utils.parse import parse_bool, parse_list
from ...utils.socket import AppriseSocketError
from ..base import NotifyBase
from . import templates
from .client import IRCClient
from .protocol import IRC_AUTH_MODES, IRCAuthMode, normalise_channel

IS_USER = re.compile(r"^\s*(@|%40)?(?P<user>[^ \t\r\n@#]+)$", re.I)

IS_CHANNEL = re.compile(
    r"^\s*(#|%23)"
    r"(?P<channel>[^ \t\r\n@#:]+)"
    r"(?::(?P<key>[^ \t\r\n]+))?\s*$",
    re.I,
)


class NotifyIRC(NotifyBase):
    """A wrapper to IRC servers using TCP or TLS."""

    # The default descriptive name associated with the Notification
    service_name = "IRC"

    # The services URL
    service_url = "https://ircv3.net/"

    # The default insecure protocol
    protocol = "irc"

    # The default secure protocol
    secure_protocol = "ircs"

    # A URL that takes you to the setup/help of the specific protocol
    setup_url = "https://appriseit.com/services/irc/"

    # RFC 2812 maximum IRC message length is 512 bytes including CRLF.
    # Keep a conservative payload budget to accommodate prefix overhead.
    body_maxlen = 380

    # IRC is not fast... there is a lot of handshaking that takes place
    # between us and the remote server. During development of this plugin
    # it took on average 18-22s to register with #EFnet; setting the value
    # to 30.0s to be conservative with others as their mileage may vary
    irc_register_timeout = 30.0

    # Avoid flooding
    request_rate_per_sec = 0.02

    # Title is prepended to body
    title_maxlen = 0

    templates = (
        "{schema}://{host}/{targets}",
        "{schema}://{host}:{port}/{targets}",
        "{schema}://{user}@{host}/{targets}",
        "{schema}://{user}@{host}:{port}/{targets}",
        "{schema}://{user}:{password}@{host}/{targets}",
        "{schema}://{user}:{password}@{host}:{port}/{targets}",
    )

    template_tokens = dict(
        NotifyBase.template_tokens,
        **{
            "host": {
                "name": _("Hostname"),
                "type": "string",
                "required": True,
            },
            "port": {
                "name": _("Port"),
                "type": "int",
                "min": 1,
                "max": 65535,
            },
            "user": {
                "name": _("User"),
                "type": "string",
            },
            "password": {
                "name": _("Password"),
                "type": "string",
                "private": True,
            },
            "target_user": {
                "name": _("Target User"),
                "type": "string",
                "prefix": "@",
                "map_to": "targets",
            },
            "target_channel": {
                "name": _("Target Channel"),
                "type": "string",
                "prefix": "#",
                "map_to": "targets",
            },
            "targets": {
                "name": _("Targets"),
                "type": "list:string",
                "required": True,
            },
        },
    )

    template_args = dict(
        NotifyBase.template_args,
        **{
            "name": {"name": _("Real Name"), "type": "string"},
            "nick": {"name": _("Nickname"), "type": "string"},
            "join": {
                "name": _("Join Channels"),
                "type": "bool",
                "default": True,
            },
            "mode": {
                "name": _("Auth Mode"),
                "type": "choice:string",
                "values": IRC_AUTH_MODES,
                "default": IRCAuthMode.SERVER,
            },
            "to": {"alias_of": "targets"},
        },
    )

    def __init__(
        self,
        targets: Optional[list[str]] = None,
        name: Optional[str] = None,
        join: Optional[bool] = None,
        nick: Optional[str] = None,
        mode: Optional[str] = None,
        **kwargs: Any,
    ) -> None:

        super().__init__(**kwargs)

        # Join Channel
        self.join = (
            parse_bool(join, self.template_args["join"]["default"])
            if join is not None
            else self.template_args["join"]["default"]
        )

        self.nickname = (nick or "").strip() or (self.user or "").strip()

        # Initialized value (as it is used in apply_irc_defaults())
        self.auth_mode = self.template_args["mode"]["default"]

        # Apply template defaults only where the user did not supply values
        self.apply_irc_defaults(**kwargs)

        if isinstance(mode, str) and mode.strip():
            self.auth_mode = mode.strip().lower()
            self.auth_mode = next(
                (a for a in IRC_AUTH_MODES if a.startswith(self.auth_mode)),
                None,
            )
            if self.auth_mode not in IRC_AUTH_MODES:
                msg = f"The IRC auth mode specified ({mode}) is invalid."
                self.logger.warning(msg)
                raise TypeError(msg)

        self.fullname = (name or "").strip()

        # For storing our channels and users to message
        self.channels: dict[str, Optional[str]] = {}
        self.users: list[str] = []

        # Set our timeouts
        srt = float(self.socket_read_timeout or 0.0)
        self.join_timeout = max(6.0, min(12.0, srt or 6.0))
        self.send_timeout = max(4.0, min(10.0, srt or 4.0))

        # Identify our targets
        self.targets = []
        for target in parse_list(targets):
            match = IS_CHANNEL.match(target)
            if match:
                channel = match.group("channel")
                key = match.group("key")
                self.channels[channel] = key
                continue

            match = IS_USER.match(target)
            if match:
                self.users.append(match.group("user"))
                continue

            self.logger.warning("Dropped invalid IRC target (%s).", target)

    def apply_irc_defaults(self, port=None, **kwargs):
        """
        A function that prefills defaults based on the irc details
        provided.
        """
        if self.port:
            # IRC port was explicitly specified, therefore it is assumed
            # the caller knows what they're doing and is intentionally
            # overriding any smart defaults that might otherwise be applied.
            return

        for i in range(len(templates.IRC_TEMPLATES)):  # pragma: no branch
            self.logger.trace(
                "Scanning %s against %s",
                self.host,
                templates.IRC_TEMPLATES[i][0],
            )

            match = templates.IRC_TEMPLATES[i][1].match(self.host)
            if match:
                self.logger.info(
                    f"Applying {templates.IRC_TEMPLATES[i][0]} Defaults"
                )

                # the secure flag can not be altered if defined in the template
                self.secure = templates.IRC_TEMPLATES[i][2].get(
                    "secure",
                    self.secure,
                )

                # store default port
                self.port = templates.IRC_TEMPLATES[i][2].get(
                    "port",
                    self.port,
                )

                self.auth_mode = templates.IRC_TEMPLATES[i][2].get(
                    "mode",
                    self.auth_mode,
                )
                break

    def send(
        self,
        body: str,
        title: str = "",
        notify_type: NotifyType = NotifyType.INFO,
        attach: Any = None,
        **kwargs: Any,
    ) -> bool:
        """Send a notification to IRC targets."""

        if not (self.channels or self.users):
            self.logger.warning("No IRC targets specified.")
            return False

        # prepare ourselves a nickname
        nickname = self.nickname or IRCClient.nick_generation(
            prefix=self.app_id,
        )

        self.throttle()

        client = IRCClient(
            host=self.host,
            nickname=nickname,
            fullname=self.fullname or self.app_desc,
            port=self.port,
            secure=self.secure,
            verify=self.verify_certificate,
            timeout=self.socket_read_timeout,
            # In ZNC mode, authentication is performed against the bouncer
            # itself. ZNC configurations expect the PASS line to include the
            # username.
            password=self.password
            if self.auth_mode != IRCAuthMode.ZNC
            else f"{self.user}:{self.password}",
            auth_mode=self.auth_mode,
            nick_generator=IRCClient.nick_generation,
        )

        try:
            client.connect()
            client.register(
                timeout=self.irc_register_timeout,
                prefix=self.app_id,
            )

            # ZNC operates as a bouncer, so perform a quick sanity check that
            # the connection is alive before issuing commands.
            if (
                self.auth_mode == IRCAuthMode.ZNC
                and not client.check_connection(
                    timeout=min(5.0, float(self.send_timeout or 5.0))
                )
            ):
                raise AppriseSocketError("ZNC connection check failed")

            message = body if not title else f"{title} {body}".strip()

            for c, key in self.channels.items():
                chan = normalise_channel(c)
                if self.join or key:
                    client.join(
                        channel=chan,
                        key=key,
                        timeout=self.join_timeout,
                        prefix=self.app_id,
                    )

                client.privmsg(
                    target=chan,
                    message=message,
                    timeout=self.send_timeout,
                )
                self.logger.info(
                    "Sent IRC notification to #%s as @%s",
                    c,
                    client.nickname,
                )

            for u in self.users:
                target = u.lstrip("@")
                client.privmsg(
                    target=target,
                    message=message,
                    timeout=self.send_timeout,
                )
                self.logger.info(
                    "Sent IRC notification to @%s as @%s",
                    u,
                    client.nickname,
                )

            client.quit(message=self.app_desc, timeout=self.send_timeout)
            return True

        except (AppriseSocketError, OSError, TimeoutError) as e:
            self.logger.warning(
                "Failed to send IRC notification to %s as @%s.",
                self.host,
                nickname,
            )
            self.logger.debug("IRC Exception: %s", e)
            return False

        finally:
            client.close()

    @property
    def url_identifier(
        self,
    ) -> tuple[str, Optional[str], Optional[str], Optional[str]]:
        """Return the pieces that uniquely identify this configuration."""
        return (
            self.secure_protocol if self.secure else self.protocol,
            self.host,
            self.user,
            self.password,
        )

    def url(self, privacy: bool = False, *args: Any, **kwargs: Any) -> str:
        """Return the URL representation of this notification."""
        params: dict[str, Any] = {
            "verify": self.verify_certificate,
            "join": self.join,
        }

        if self.auth_mode and self.auth_mode != IRCAuthMode.SERVER:
            params["mode"] = self.auth_mode

        if self.fullname:
            params["name"] = self.fullname

        if self.nickname and self.nickname != (self.user or ""):
            params["nick"] = self.nickname

        params.update(self.url_parameters(privacy=privacy, *args, **kwargs))

        auth = ""
        if self.user and self.password:
            auth = "{user}:{password}@".format(
                user=self.quote(self.user, safe=""),
                password=self.pprint(
                    self.password,
                    privacy,
                    mode=PrivacyMode.Secret,
                    safe="",
                ),
            )
        elif self.user:
            auth = "{user}@".format(user=self.quote(self.user, safe=""))

        default_port = (
            IRCClient.default_secure_port
            if self.secure
            else IRCClient.default_insecure_port
        )

        port = (
            self.port
            if isinstance(self.port, int)
            else (
                IRCClient.default_secure_port
                if self.secure
                else IRCClient.default_insecure_port
            )
        )

        port = "" if port == default_port else f":{port}"

        schema = self.secure_protocol if self.secure else self.protocol
        return "{schema}://{auth}{host}{port}/{targets}?{params}".format(
            schema=schema,
            auth=auth,
            host=self.host,
            port=port,
            targets="/".join(
                chain(
                    [
                        self.quote(
                            f"#{c}"
                            if not k
                            else "#{}:{}".format(
                                c,
                                self.pprint(
                                    k,
                                    privacy,
                                    mode=PrivacyMode.Secret,
                                    safe="",
                                ),
                            ),
                            safe="#",
                        )
                        for c, k in self.channels.items()
                    ],
                    [self.quote(f"@{u}", safe="@") for u in self.users],
                )
            ),
            params=self.urlencode(params),
        )

    @staticmethod
    def parse_url(url: str) -> Optional[dict[str, Any]]:
        """Parse an IRC URL into constructor arguments."""
        results = NotifyBase.parse_url(url, verify_host=False)
        if not results:
            return None

        results["targets"] = []

        if "host" in results["qsd"] and len(results["qsd"]["host"]):
            # a host was defined which means the first entry is actually one
            # of our targets
            results["targets"].append(NotifyIRC.unquote(results["host"]))

        # Store remaining targets
        results["targets"].extend(NotifyIRC.split_path(results["fullpath"]))

        # The 'to' makes it easier to use yaml configuration
        if "to" in results["qsd"] and len(results["qsd"]["to"]):
            results["targets"] += NotifyIRC.parse_list(
                NotifyIRC.unquote(results["qsd"]["to"])
            )

        # Get Join Channel Flag
        results["join"] = parse_bool(
            results["qsd"].get(
                "join", NotifyIRC.template_args["join"]["default"]
            )
        )

        # Get our IRC Name
        if "name" in results["qsd"] and len(results["qsd"]["name"]):
            results["name"] = NotifyIRC.unquote(results["qsd"]["name"])

        if "nick" in results["qsd"] and len(results["qsd"]["nick"]):
            results["nick"] = NotifyIRC.unquote(results["qsd"]["nick"])

        if "mode" in results["qsd"] and len(results["qsd"]["mode"]):
            results["mode"] = NotifyIRC.unquote(results["qsd"]["mode"])

        return results



# BSD 2-Clause License
#
# Apprise - Push Notification Library.
# Copyright (c) 2026, Chris Caron <lead2gold@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import json

# Disable logging for a cleaner testing output
import logging
from unittest import mock

from helpers import AppriseURLTester
import requests

from apprise import Apprise
from apprise.plugins.home_assistant import NotifyHomeAssistant

logging.disable(logging.CRITICAL)

# Our Testing URLs
apprise_url_tests = (
    (
        "hassio://:@/",
        {
            "instance": TypeError,
        },
    ),
    (
        "hassio://",
        {
            "instance": TypeError,
        },
    ),
    (
        "hassios://",
        {
            "instance": TypeError,
        },
    ),
    # No Long Lived Access Token specified
    (
        "hassio://user@localhost",
        {
            "instance": TypeError,
        },
    ),
    (
        "hassio://localhost/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://localhost/prefix/path/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://localhost/long.lived.token?prefix=/ha",
        {
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://localhost/service/?token=long.lived.token&prefix=/ha",
        {
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://localhost/?token=long.lived.token&prefix=/ha&to=service",
        {
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://localhost/service/$%/?token=long.lived.token&prefix=/ha",
        {
            # Tests an invalid service entry
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://localhost/%only%/%invalid%/?token=lng.lived.token&prefix=/ha",
        {
            # Tests an invalid service entry
            "instance": NotifyHomeAssistant,
            # we'll have a notify response failure in this case
            "notify_response": False,
        },
    ),
    (
        "hassio://localhost/service/?accesstoken=long.lived.token",
        {
            # accesstoken via query string (?accesstoken=)
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://user:pass@localhost/long.lived.token/",
        {
            "instance": NotifyHomeAssistant,
            # Our expected url(privacy=True) startswith() response:
            "privacy_url": "hassio://user:****@localhost/l...n",
        },
    ),
    (
        "hassio://localhost:80/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassio://user@localhost:8123/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
            "privacy_url": "hassio://user@localhost/l...n",
        },
    ),
    (
        "hassios://localhost/long.lived.token?nid=!%",
        {
            # Invalid notification_id
            "instance": TypeError,
        },
    ),
    (
        "hassios://localhost/long.lived.token?nid=abcd",
        {
            # Valid notification_id
            "instance": NotifyHomeAssistant,
        },
    ),
    (
        "hassios://user:pass@localhost/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
            "privacy_url": "hassios://user:****@localhost/l...n",
        },
    ),
    (
        "hassios://localhost:8443/path/long.lived.token/",
        {
            "instance": NotifyHomeAssistant,
            "privacy_url": "hassios://localhost:8443/l...n",
        },
    ),
    (
        "hassio://localhost:8123/a/path?token=long.lived.token",
        {
            "instance": NotifyHomeAssistant,
            # Default port; so it's stripped off
            # token was specified as kwarg
            "privacy_url": "hassio://localhost/l...n",
        },
    ),
    (
        "hassios://user:password@localhost:80/long.lived.token/",
        {
            "instance": NotifyHomeAssistant,
            "privacy_url": "hassios://user:****@localhost:80",
        },
    ),
    (
        "hassio://user:pass@localhost:8123/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
            # force a failure
            "response": False,
            "requests_response_code": requests.codes.internal_server_error,
        },
    ),
    (
        "hassio://user:pass@localhost/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
            # throw a bizzare code forcing us to fail to look it up
            "response": False,
            "requests_response_code": 999,
        },
    ),
    (
        "hassio://user:pass@localhost/long.lived.token",
        {
            "instance": NotifyHomeAssistant,
            # Throws a series of connection and transfer exceptions
            # when this flag is set and tests that we gracfully handle them
            "test_requests_exceptions": True,
        },
    ),
)


def test_plugin_homeassistant_urls():
    """NotifyHomeAssistant() Apprise URLs."""

    # Run our general tests
    AppriseURLTester(tests=apprise_url_tests).run_all()


@mock.patch("requests.post")
def test_plugin_homeassistant_general(mock_post):
    """NotifyHomeAssistant() General Checks."""

    response = mock.Mock()
    response.content = ""
    response.status_code = requests.codes.ok

    # Prepare Mock
    mock_post.return_value = response

    # Variation Initializations
    obj = Apprise.instantiate("hassio://localhost/accesstoken")
    assert isinstance(obj, NotifyHomeAssistant) is True
    assert isinstance(obj.url(), str) is True

    # Send Notification
    assert obj.send(body="test") is True

    assert mock_post.call_count == 1
    assert (
        mock_post.call_args_list[0][0][0]
        == "http://localhost:8123/api/services/persistent_notification/create"
    )

    # Reset our mock object
    mock_post.reset_mock()

    # Now let's notify an object
    obj = Apprise.instantiate("hassio://localhost/long.lived.token/service")
    assert isinstance(obj, NotifyHomeAssistant) is True
    assert isinstance(obj.url(), str) is True

    # Send Notification
    assert obj.send(body="test") is True

    assert (
        mock_post.call_args_list[0][0][0]
        == "http://localhost:8123/api/services/notify/service"
    )
    posted_json = json.loads(mock_post.call_args_list[0][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" not in posted_json
    assert "message" in posted_json
    assert posted_json["message"] == "test"
    assert "title" in posted_json
    assert posted_json["title"] == ""

    # Reset our mock object
    mock_post.reset_mock()

    #
    # No Batch Processing
    #

    # Now let's notify an object
    obj = Apprise.instantiate(
        "hassio://localhost/long.lived.token/serviceA:target1,target2/"
        "service2/domain1.service3?batch=no"
    )
    assert isinstance(obj, NotifyHomeAssistant) is True
    assert isinstance(obj.url(), str) is True

    # Send Notification
    assert obj.send(body="test-body", title="title") is True

    # Entries are split apart
    assert len(obj) == 4
    assert mock_post.call_count == 4

    assert (
        mock_post.call_args_list[0][0][0]
        == "http://localhost:8123/api/services/domain1/service3"
    )
    posted_json = json.loads(mock_post.call_args_list[0][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" not in posted_json
    assert "message" in posted_json
    assert posted_json["message"] == "test-body"
    assert "title" in posted_json
    assert posted_json["title"] == "title"

    assert (
        mock_post.call_args_list[1][0][0]
        == "http://localhost:8123/api/services/notify/service2"
    )
    posted_json = json.loads(mock_post.call_args_list[1][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" not in posted_json
    assert "message" in posted_json
    assert posted_json["message"] == "test-body"
    assert "title" in posted_json
    assert posted_json["title"] == "title"

    assert (
        mock_post.call_args_list[2][0][0]
        == "http://localhost:8123/api/services/notify/serviceA"
    )
    posted_json = json.loads(mock_post.call_args_list[2][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" in posted_json
    assert isinstance(posted_json["targets"], list)
    assert len(posted_json["targets"]) == 1
    assert "target1" in posted_json["targets"]
    assert "message" in posted_json
    assert posted_json["message"] == "test-body"
    assert "title" in posted_json
    assert posted_json["title"] == "title"

    assert (
        mock_post.call_args_list[3][0][0]
        == "http://localhost:8123/api/services/notify/serviceA"
    )
    posted_json = json.loads(mock_post.call_args_list[3][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" in posted_json
    assert isinstance(posted_json["targets"], list)
    assert len(posted_json["targets"]) == 1
    assert "target2" in posted_json["targets"]
    assert "message" in posted_json
    assert posted_json["message"] == "test-body"
    assert "title" in posted_json
    assert posted_json["title"] == "title"

    # Reset our mock object
    mock_post.reset_mock()

    #
    # Batch Processing
    #

    # Now let's notify an object
    obj = Apprise.instantiate(
        "hassio://localhost/long.lived.token/serviceA:target1,target2/"
        "service2/domain1.service3?batch=yes"
    )
    assert isinstance(obj, NotifyHomeAssistant) is True
    assert isinstance(obj.url(), str) is True

    # Send Notification
    assert obj.send(body="test-body", title="title") is True

    # Entries targets can be grouped
    assert len(obj) == 3
    assert mock_post.call_count == 3

    assert (
        mock_post.call_args_list[0][0][0]
        == "http://localhost:8123/api/services/domain1/service3"
    )
    posted_json = json.loads(mock_post.call_args_list[0][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" not in posted_json
    assert "message" in posted_json
    assert posted_json["message"] == "test-body"
    assert "title" in posted_json
    assert posted_json["title"] == "title"

    assert (
        mock_post.call_args_list[1][0][0]
        == "http://localhost:8123/api/services/notify/service2"
    )
    posted_json = json.loads(mock_post.call_args_list[1][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" not in posted_json
    assert "message" in posted_json
    assert posted_json["message"] == "test-body"
    assert "title" in posted_json
    assert posted_json["title"] == "title"

    assert (
        mock_post.call_args_list[2][0][0]
        == "http://localhost:8123/api/services/notify/serviceA"
    )
    posted_json = json.loads(mock_post.call_args_list[2][1]["data"])
    assert "notification_id" not in posted_json
    assert "targets" in posted_json
    assert isinstance(posted_json["targets"], list)
    # Our batch groups our targets
    assert len(posted_json["targets"]) == 2
    assert "target1" in posted_json["targets"]
    assert "target2" in posted_json["targets"]
    assert "message" in posted_json
    assert posted_json["message"] == "test-body"
    assert "title" in posted_json
    assert posted_json["title"] == "title"

    # Reset our mock object
    mock_post.reset_mock()

    #
    # Test error handling on multi-query request
    #

    # Now let's notify an object
    obj = Apprise.instantiate(
        "hassio://localhost/long.lived.token/serviceA:target1,target2/"
        "service2:target3,target4,target5,target6?batch=no"
    )

    assert isinstance(obj, NotifyHomeAssistant) is True
    assert isinstance(obj.url(), str) is True

    bad_response = mock.Mock()
    bad_response.content = ""
    bad_response.status_code = requests.codes.not_found

    mock_post.side_effect = (response, bad_response)

    # We will fail on our second message sent
    assert obj.send(body="test-body", title="title") is False
